﻿using System;
using System.IO;

namespace MaszynaTuringa
{
    public static class TuringHelper
    {
        public static void ConsoleWriteLine(this Turing turing)
        {
            Console.WriteLine("Stan głowicy: " + turing.StanGłowicy);
            Console.WriteLine("Taśma: " + new string(turing.Taśma));
            Console.WriteLine("Program: ");
            foreach (string linia in turing.Program) Console.WriteLine(linia);
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Plik wykonywalny: " + System.Reflection.Assembly.GetExecutingAssembly().Location);
            Console.WriteLine("Katalog roboczy: " + Environment.CurrentDirectory);
            Console.WriteLine("PATH: " + Environment.GetEnvironmentVariable("PATH"));

            Console.WriteLine("Argumenty linii komend:");
            foreach (string arg in args)
                Console.WriteLine(arg);
            Console.WriteLine();

            if (args.Length != 2)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.Error.WriteLine("Nieprawidłowa liczba argumentów");
                return;
            }

            string ścieżkaPlikuProgram = args[0];
            string ścieżkaPlikuTaśma = args[1];            

            if(!File.Exists(ścieżkaPlikuTaśma))
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.Error.WriteLine("Brak pliku z taśmą: " + ścieżkaPlikuTaśma);
                return;
            }

            //string[] kodProgramu = File.ReadAllLines("program.txt");
            //string taśma = File.ReadAllText("taśma.txt");
            string[] kodProgramu = File.ReadAllLines(ścieżkaPlikuProgram);
            string taśma = File.ReadAllText(ścieżkaPlikuTaśma);

            /*
            string[] kodProgramu =
            {
                "aABb",
                "bBRb",
                "bAAa"
            };
            string taśma = "aAABCAD";
            */

            Console.WriteLine("Początkowy stan maszyny: " + taśma);
            Console.WriteLine("Program: ");
            foreach (string linia in kodProgramu) Console.WriteLine(linia);

            Turing turing = new Turing(taśma, kodProgramu);
            //TuringHelper.ConsoleWriteLine(turing);
            turing.ConsoleWriteLine();

            turing.ZmianaStanu += 
                (object sender, Turing.TuringEventArgs e) => 
                {
                    //Console.WriteLine("Nastąpiła zmiana stanu");
                    Console.WriteLine(e.BieżącyStan);
                };

            Console.WriteLine("\nWynik działania programu: ");
            turing.Run();
            //Console.WriteLine(turing.Output.Length.ToString());            
            //foreach (string s in turing.Output) Console.WriteLine(s);
        }
    }
}
